# Step 1: Extracting Census Data

Extract data from the Census Reporter API.

This task is a bit more complicated than it sounds,
since we are interested in so many different quantities.
In order to know how to call the API,
and what to do with the results it returns,
we have to decide on a data scheme.

## MongoDB

As a brief aside, to run mongodb,

```
$ mongod --config ./mongod.conf
```

## Census Reporter API (Data Source)

The Census Reporter API can be called a variety of different ways.
We will make two different kinds of calls: one for the geometry of different
geographic entities, and one for census data tables.

The data will be used for two distinct purposes: 
the geometry information will be used for maps (with Leaflet.js), 
and the properties or census data tables will be used to build charts (with D3.js).

### The Geometry Table (Maps)

**Maps** - to draw a map with Leaflet, we need to have a large amount of 
information about polygons and points.

This is the information returned by the Census Reporter's
geo information API:

```
http://api.censusreporter.org/1.0/geo/show/tiger2013?geo_ids=GEOID"
```

where ```GEOID``` contains information about 
the level of information requested, as well as 
the parent geographic entity. 

In our case, our ```GEOID``` looks like this:

```
GEOID = "140|31000US35620"
```

Here, 140 indicates we want to receive polygons for 
census tracts. (These numbers come from the US Census Bureau's
[Census Data Technical Manual](http://www.census.gov/prod/cen2010/doc/sf1.pdf).)

310 indicates we want to specify the parent geographic entity 
to be a metropolitan area. These are indexed using something
called a CSBA - core based statistical area. You can find a 
huge list of CSBAs for cities in the United States 
[on the Census Bureau website](http://www.census.gov/population/metro/data/def.html).
The master list is contained in a single Excel spreadsheet.

This API URL can now be used to obtain GeoJSON information containing
the polygons that make up each census tract in the metro area 
we have specified (New York, in the example above).

This information will compose our first table,
**THE GEOMETRY TABLE.**

### The Properties Table (Charts)

**Charts** - we will be creating charts with D3.js 
using properties contained in each geographic entity.

```
add api endpoint here
```


## The MongoDB Database

We will be using MongoDB on the backend to store all of
this data and make it available for our statistical modeling
endeavors.

### The MongoDB Schema

The MongoDB database will have two tables, corresponding
to the two types of data:

* Geometry table - stores information about geometry (but not properties) of geographic entities
* Properties table - stores properties (but not geometry) of each geographic entity

This separation allows for faster access for the D3 charts.

## Geometry 

The geometry table is implemented as a pair:
the geometry table, and the geometry meta-table.

Currently, the geometry meta-table is not used, but it would 
contain mappings of codes to names, and store information
common to all records, to reduce redundancy and storage.

### Geometry Table: Storing GeoJson with MongoDB

In order to store the data in this split way,
and to make it easier to index,
we format the GeoJson a bit differently than it
is returned by the Census Reporter REST API.

The Census Reporter REST API will return geo data 
that looks like this:

```
{
    "type" : "MultiPolygon",
    "features" : [
                    {"geometry" : ...,
                     "type" : ...,
                     "properties" : ...
                    },
                    {"geometry" : ...,
                     "type" : ...,
                     "properties" : ...
                    },
                    ...
                ]
}
```

(Here, properties is basically empty, but when we get data
from census data tables, this field will contain the resulting 
table quantities.)

What we want to do is to wrap the MongoDB documents
to be the dictionary array contained in "features".

What we'll do is, store each of those dictionaries 
(indexed by geoid, geometry, type, and properties) in the
MongoDB Geometry table. When we search for records,
we will pass search criteria, and MongoDB will return
a list of features.

We can then wrap that call to MongoDB 
and put the results into a dictionary 
with the "type" and "features" keys,
which is the format that Leaflet.js expects.

### Geometry Table: Pseudocode

Some pseudocode for the above operation:

```
getJsonFromMongo(search_parameters):
    
    d = {}
    d['type'] = 'MutiPolygon'

    for record in mongodb.search():

        # here, record contains ["geometry","type","properties"]
        features.append( record )
    
    d['features'] = features

    return d
```

### Geometry Table: Schema

The geometry table contains a set of entities with the following key structure:

```
{
    _id : automatically assigned
    geoid : <geoid>
    metroid: <metroid>
    geometry : <geometry>
    type: <type>
    properties : <properties>

    metroname: <metroname>
    name : <name>
}
```

This will be populated using data from census data tables, contained in the 
Properties collection of our MongoDB. But this is not done in the database
because we want the flexibility of storing these two sets of information 
separately.

### Result of Geometry Table Population Script

```
$ python GetMetro.py
Doing Houston
Calling Census Reporter API...
Calling API took 7.176 seconds
Inserting record into database...
Calling insert on 1102 records took 0.130 seconds
Done
Doing Phoenix
Calling Census Reporter API...
Calling API took 4.498 seconds
Inserting record into database...
Calling insert on 1015 records took 0.100 seconds
Done
Doing Dallas
Calling Census Reporter API...
Calling API took 8.040 seconds
Inserting record into database...
Calling insert on 1360 records took 0.164 seconds
Done
Doing Miami
Calling Census Reporter API...
Calling API took 4.413 seconds
Inserting record into database...
Calling insert on 1237 records took 0.102 seconds
Done
Doing Boston
Calling Census Reporter API...
Calling API took 6.564 seconds
Inserting record into database...
Calling insert on 1085 records took 0.137 seconds
Done
Doing Chicago
Calling Census Reporter API...
Calling API took 7.669 seconds
Inserting record into database...
Calling insert on 2275 records took 0.172 seconds
Done
Doing Philadelphia
Calling Census Reporter API...
Calling API took 7.966 seconds
Inserting record into database...
Calling insert on 1558 records took 0.175 seconds
Done
Doing Detroit
Calling Census Reporter API...
Calling API took 4.271 seconds
Inserting record into database...
Calling insert on 1353 records took 0.100 seconds
Done
Doing New York
Calling Census Reporter API...
Calling API took 14.501 seconds
Inserting record into database...
Calling insert on 4804 records took 0.432 seconds
Done
Doing Los Angeles
Calling Census Reporter API...
Calling API took 10.063 seconds
Inserting record into database...
Calling insert on 2978 records took 0.261 seconds
Done
Doing Washington DC
Calling Census Reporter API...
Calling API took 11.614 seconds
Inserting record into database...
Calling insert on 1440 records took 0.231 seconds
Done
Doing San Francisco
Calling Census Reporter API...
Calling API took 5.973 seconds
Inserting record into database...
Calling insert on 1018 records took 0.131 seconds
Done
Doing Atlanta
Calling Census Reporter API...
Calling API took 8.746 seconds
Inserting record into database...
Calling insert on 1019 records took 0.163 seconds
Done
Doing Seattle
Calling Census Reporter API...
Calling API took 5.916 seconds
Inserting record into database...
Calling insert on 747 records took 0.094 seconds
Done
Doing Riverside
Calling Census Reporter API...
Calling API took 8.043 seconds
Inserting record into database...
Calling insert on 882 records took 0.098 seconds
Done
Done populating geometry table
```

### Geometry Meta Table

As mentioned above, each table is implemented as a pair.
The Geometry table has a corresponding Geometry Meta table,
which is used to translate the metropolitan area CSBA codes
and the census tract geoids into string names, but while 
avoiding the overhead of storing highly redundant information 
with each record (which is how MongoDB stores it).



## Properties 

Properties table is implemented as a pair:
the properties table, and the properties meta-table.

### Properties Table: Storing with MongoDB

As above, some rearrangement from the Json format returned by the 
Census Reporter API is needed to store data in a split way,
indexed by geoid. 

The Census Reporter REST API will return table data 
that looks like this:

```
{
    "release"   : ...
    "tables"    : ...
    "data"      : ...
    "geography" : ...
}
```

The two pieces of this Json that we are most interested in
are "tables" and "data".

The property "tables" contains a set of keys for column headers - 
without these, the numbers in the data tables don't have context.
This information goes into the properties meta table,
because it is common to all variables.

The property "data" contains all of the data in a dictionary 
(hash), indexed by their geoids. When you pass in a geoid index,
you are returned an estimate and an error for each quantity 
in the table. This goes into the properties table.

## Properties Table: Schema

The property data is stored and indexed by its geoid. Each geoid
has a value for each variable in the table:

```
geoid   var1    var2    var3    var4
...     ...     ...     ...     ...
...     ...     ...     ...     ...
...     ...     ...     ...     ...
...     ...     ...     ...     ...
```

As we continue to add more data from more tables,
we keep adding columns.

The Census Reporter REST API returns a dictionary that
looks like this:

```
{
    "tables" : ...,

    "data" : {
        "14000US48251130304" : {
            "B15002" : {
                "estimate" : {
                    "B15002022" : 25.0,
                    "B15002025" : 8.0,
                    ...
                },
                "error" : {
                    "B15002022" : 1.0,
                    "B15002025" : 0.2,
                    ...
                }
            }
        }
    }

    "release" : ...,

    "geography" : ...
}
```

To put this into GeoJson, we extract the values for each column.
We don't need to keep the structure. We'll reformat this into:

```
{
    _id : <geoid>
    geoid : <geoid>
    B15002022 : ...
    B15002023 : ...
    B15002024 : ...
    B15002025 : ...

    ...

    B15002022err : ...
    B15002023err : ...
    B15002024err : ...
    B15002025err : ...
}
```



## Properties Meta-Table

The property tables column headers are stored in their own table.
Each table of properties that is added to the database has a new
entry added to the properties meta table, with the following
key/value attributes defined:


```
code                    name
<tablecode>             <Title of table>
<tablecode>universe     <Universe of table>
<tablecode>yr           <Year of table data>
<column 1 name>         <Long name of column 1>
<column 2 name>         <Long name of column 2>
[...]
<column N name>         <Long name of column N>
```

In addition, because we might have inconsistencies 
between which cities have which tables, 
(e.g., we have table B15001 for Houston and B17320 for Dallas),
we add some geoid information to the properties
meta table as well:

```
geoid           tables
<geoid>         {
                    <tableid> : True,
                    <tableid> : True,
                    <tableid> : True
                }
```

(NOTE: This is only done for the metro level,
not for the individual census tract level.)

Now we can use the properties meta table 
to check if we've already added a particular table
for a particular metro area
by getting the document with key "geoid" 
equal to that metro area's geoid,
and then check if the keys of the tables value 
contain the requested table id.

The Census Reporter REST API returns a dictionary that
looks like this:

```
{
    "tables" : {
        "B15002" : {
            "universe" : ...
            "columns"  : ...,
            "title"    : ...,
            "denominator_column_id" : ...
        }
    },

    "data" : ...,

    "release" : ...,

    "geography" : ...
}
```

We extract the tables information,
store the universe/title, and then
go through the columns and add their
codes and full names.


## Summary of Files

Files are grouped/listed by functionality:

**API Call and MongoDB Population:**

* GetMetro.py - the workhorse script that interacts with the Census Reporter API,
formats the resulting JSON, and populates the MongoDB with it

**Utility:**

* MetroCodes.py - utility script for populating a JSON file with metro codes (CSBAs - see above)
* FIPSCodes.py - utility for turning state abbreviations into their FIPS codes. Not used.

**Testing:**

* MongoMetro.py - testing creating mongo db tables/collections

**Data/Misc Files:**

* metros.json - contains city CSBAs; created by MetroCodes.py
* fips.json - contains state FIPS codes
* mongod.conf - config file for running mongodb

