from timeit import default_timer as timer
import json
import urllib2

import numpy as np
import pandas as pd

import re
import os
import subprocess

from collections import OrderedDict
from pymongo import MongoClient

from Lumberjack import *
from Codes import CBSACode
from Codes import FIPSCode
from . import Mogo


########################################
# Metro-specific operations 
# for populating data into MongoDB
# via Census Reporter API

class MetroMogo(Mogo):
    """
    defines Metro-specific operations
    for populating data into this database
    """

    ############################################
    # API Call methods

    def get_metro_geodata(self,metroid):
        """
        Get GeoJson for metro area
        """
        target_geoid = self.metro_level + "00" + "US" + str(metroid)
        return self.get_geodata(target_geoid)

    def get_metro_county_geodata(self,metroid):
        """
        Get GeoJson for census tracts in a metro area
        """
        target_geoid = self.county_level + "|" + self.metro_level + "00" + "US" + str(metroid)
        return self.get_geodata(target_geoid)

    def get_metro_census_geodata(self,metroid):
        """
        Get GeoJson for census tracts in a metro area
        """
        target_geoid = self.censustract_level + "|" + self.metro_level + "00" + "US" + str(metroid)
        return self.get_geodata(target_geoid)

    def get_geodata(self,target_geoid):
        """
        Call the Census Reporter Geo API endpoint
        Returns GeoJson
        """
        api_call = re.sub('GEOID',target_geoid,self.geo_api_url)

        start_api = timer()
        metros = json.load(urllib2.urlopen(api_call))
        end_api = timer()

        logging.info("Calling API took %0.3f seconds"%(end_api-start_api))

        return metros

    def get_metro_tabledata(self,tablecode,metroid):
        """
        Get properties from tablecode for metro area
        """
        target_geoid = self.metro_level+"00"+"US"+str(metroid)  
        return self.get_tabledata(tablecode,target_geoid)

    def get_metro_county_tabledata(self,tablecode,metroid):
        """
        Get properties from tablecode for 
        counties in a given metro area
        """
        target_geoid = self.county_level + "|" + self.metro_level + "00" + "US" + str(metroid)
        return self.get_tabledata(tablecode,target_geoid)

    def get_metro_census_tabledata(self,tablecode,metroid):
        """
        Get properties from tablecode for 
        census tracts in a given census tract metro area
        """
        target_geoid = self.censustract_level + "|" + self.metro_level + "00" + "US" + str(metroid)
        return self.get_tabledata(tablecode,target_geoid)

    def get_tabledata(self,tablecode,target_geoid):
        """
        Call the Census Reporter Geo API endpoint
        Returns nested dictionaries 
        """
        api_call = re.sub('GEOID',target_geoid,self.table_api_url)
        api_call = re.sub('TABLEID',tablecode,api_call)

        start_api = timer()
        tables = json.load(urllib2.urlopen(api_call))
        end_api = timer()

        logging.info("Calling API took %0.3f seconds"%(end_api-start_api))

        return tables




    ############################################
    # Add Geometry/Properties methods 

    def add_geometry(self, CBSAcode, name):
        """
        Add census tracts and metro area
        for the metro area with this CBSA/name
        to the geometry database.
        """

        # check if this is in geometry table already
        if(self.GeometryMeta.find({'metroid':CBSAcode}).count() > 0):
            msg = "Database already has geometry data for metroid %s (%s)"%(CBSAcode,name)
            logging.info(msg)
            return

        logging.info("Adding metro %s (%s) to the geometry table"%(name,CBSAcode))


        def meta_geometry_entry(feature_data):
            """
            Return a geometry meta table
            from a feature data set
            """
            geometry_meta_table = []
            for feature in feature_data:
                d={}
                d['metroid']   = CBSAcode
                d['metroname'] = name
                d['geoid']     = feature['properties']['geoid']
                d['name']      = feature['properties']['name']
                geometry_meta_table.append(d)

            return geometry_meta_table


        # -----------------
        # First API call:
        # metropolitan entity itself
        metros = self.get_metro_geodata(CBSAcode)

        # Insert feature data into geometry meta table
        feature_data = metros['features']
        geometry_meta_table = meta_geometry_entry(feature_data)
        self.insert_geometry_meta(geometry_meta_table)

        # Add useful indexes to geometry, then add geom to MongoDB 
        for feature in feature_data:
            # add these indexes to make 
            # MongoDB index/search easier 
            feature['metroid'] = CBSAcode
            feature['geoid']   = feature['properties']['geoid']
        self.insert_geometry(feature_data)


        # -----------------
        # Second API call:
        # all census tracts in the metropolitan entity
        metros = self.get_metro_census_geodata(CBSAcode)

        # Insert feature data into geometry meta table
        feature_data = metros['features']
        geometry_meta_table = meta_geometry_entry(feature_data)
        self.insert_geometry_meta(geometry_meta_table)

        # Add useful indexes to geometry, then add geom to MongoDB 
        for feature in feature_data:
            # add these to use for MongoDB indexing 
            feature['metroid'] = CBSAcode
            feature['geoid']   = feature['properties']['geoid']
        self.insert_geometry(feature_data)

        logging.info("Done adding geometry for %s (%s) to geometry database."%(name,CBSAcode))



    def add_property(self, tablecode, CBSAcode, name):
        """
        Add census table data for metro area with this 
        CBSA/name to the property database.
        """

        # check if this is in property table already
        # 
        # if tableid is in properties meta table,
        #     if geoid is in properties table
        #
        # fix this: we updated propertiesmeta to contain geoid-table

        #if( self.PropertiesMeta.find({'tableid':tablecode}).count() > 0 
        #and self.Properties.find({'geoid':CBSAcode}).count() > 0 ):
        #    logging.info("Database already has property data for metroid",CBSAcode,"(",name,")")
        #    return

        if(self.PropertiesMeta.find({'geoid':CBSAcode}).count() > 0):
            d = self.PropertiesMeta.find_one({'geoid':CBSAcode})
            if tablecode in d['tables']:
                msg = "Database already has property data for table %s and metroid %s (%s)"%(tablecode,CBSAcode,name)
                logging.info(msg)
                return


        logging.info("Adding table %s for metro %s (%s) to the property table"%(tablecode,name,CBSAcode))

        def meta_properties_columns_entry(tables):
            """
            Return a properties meta table 
            from census table data
            """
            table_data = tables['tables'][tablecode]

            properties_meta_table = []

            d = {}
            d['code'] = tablecode
            d['name'] = table_data['title']
            properties_meta_table.append(d)

            d = {}
            d['code'] = tablecode+"universe"
            d['name'] = table_data['universe']
            properties_meta_table.append(d)

            datacols = table_data['columns']
            for colid,colvalue in zip(datacols.keys(),datacols.values()):
                d = {}
                d['code'] = colid
                d['name'] = colvalue['name']
                properties_meta_table.append(d)

            d = {}
            d['code'] = tablecode+"yr"
            d['name'] = tables['release']['years']
            properties_meta_table.append(d)

            # Now put it in the properties table
            self.insert_properties_meta(properties_meta_table)



        def meta_properties_geoid_entry(this_geoid,levelcode):

            # check if this metro area already in here
            if( self.PropertiesMeta.find({'geoid':this_geoid}).count() > 0 ):
                
                g = self.PropertiesMeta.find_one({'geoid':this_geoid})
                ts = g['tables']
                ts = ts + [tablecode]

                d = {}
                d['geoid'] = this_geoid
                d['level'] = levelcode
                d['tables'] = ts

                self.PropertiesMeta.find_and_modify( query = {'geoid':this_geoid},
                                                     update = {"$set": {'tables':ts} }, 
                                                     upsert = False, 
                                                     full_response = True)
            else:
                d = {}
                d['geoid'] = this_geoid
                d['level'] = levelcode
                d['tables'] = [tablecode]

                my_id = self.PropertiesMeta.insert(d)




        def properties_entry(tables):

            table_data = tables['data']
            geoids = table_data.keys()

            properties_table = []
            for this_geoid in geoids:
                props_dict = {}
                props_dict['metroid'] = CBSAcode
                props_dict['geoid'] = this_geoid

                this_table = tables['data'][this_geoid][tablecode]

                estimate_values = this_table['estimate']
                for estkey in estimate_values.keys():
                    props_dict[estkey] = estimate_values[estkey]

                #error_values    = tables['data'][this_geoid][table_id]['error']
                #for errkey in error_values.keys():
                #    props_dict[errkey+"err"] = error_values[errkey]

                if( self.Properties.find({'geoid':this_geoid}).count() > 0 ):

                    self.Properties.find_and_modify( query = {'geoid': this_geoid},
                                                     update = {"$set": props_dict }, 
                                                     upsert = True, 
                                                     full_response = True)

                else:
                    self.Properties.insert(props_dict)

                properties_table.append(props_dict)



        # -----------------
        # First API call:
        # metropolitan entity itself
        tables = self.get_metro_tabledata(tablecode,CBSAcode)

        # Insert properties meta data 
        # into properties meta table
        # (do this second, in case the first opertation fails)
        meta_properties_columns_entry(tables)
        meta_properties_geoid_entry(CBSAcode,self.metro_level)

        # Insert properties into 
        # properties table
        properties_entry(tables)


        # -----------------
        # Second API call:
        # all counties in metropolitan entity
        tables = self.get_metro_county_tabledata(tablecode,CBSAcode)

        properties_entry(tables)



        # -----------------
        # Third API call:
        # all census tracts in the metropolitan entity
        tables = self.get_metro_census_tabledata(tablecode,CBSAcode)

        # Insert properties into 
        # properties table
        properties_entry(tables)


        logging.info("Done adding table data for table %s for %s (%s) to properties database."%(tablecode,name,CBSAcode))


    def get_df(self,city,tablecode=None):
        """
        Return a dataframe with
        every geoid that is in city
        (not including city itself)
        """

        metaprops = self.PropertiesMeta
        props = self.Properties 

        #########################
        # Mongo lookup
        
        cbsa = CBSACode(city)[0]
        
        pre_search = metaprops.find_one({'geoid':cbsa})

        if tablecode is not None:
            if tablecode not in pre_search['tables']:
                print "Error: could not find table",tablecode,"for city",city,"in db."

        mongo_search = props.find({'$and': [
                                            {'metroid':cbsa},
                                            {'geoid':{'$nin':[cbsa]}}
                                            ]
                                })

        if mongo_search.count()==0:
            print "Error: could not find city",city,"in db."
        
        df = pd.DataFrame([])
        srch = list(mongo_search)

        for i,r in enumerate(srch):
            if cbsa in r['geoid']:
                del srch[i]

        if tablecode is not None:
            for i,r in enumerate(srch):
                for k in r.keys():
                    if (tablecode not in k) and (k<>'geoid'):
                        del r[k]

        df = df.append(srch)

        return df

